package com.qtzl.sample.manager.sf;

import cn.com.taiji.common.manager.AbstractManager;
import cn.com.taiji.common.pub.StringTools;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.jcraft.jsch.SftpException;
import com.qtzl.sample.dao.repo.jpa.SfRepo;
import com.qtzl.sample.manager.model.SfModel;
import com.qtzl.sample.manager.model.VehicleModel;
import com.qtzl.sample.manager.tools.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.*;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class SFFileManagerImpl extends AbstractManager implements SFFileManager {

    /** szftp总流水服务ip地址（工行） */
    @Value("${commProperties.ftp_sz_icbc_ip_addreess}")
    private String icbcZlsIpAddreess;
    /** szftp总流水服务端口（工行） */
    @Value("${commProperties.ftp_sz_icbc_port}")
    private Integer icbcZlsPort;
    /** szftp总流水服务账号（工行） */
    @Value("${commProperties.ftp_sz_icbc_name}")
    private String icbcZlsName;
    /** szftp总流水服务密码（工行） */
    @Value("${commProperties.ftp_sz_icbc_pwd}")
    private String icbcZlsPwd;

    /** szftp签约数据文件存放目录（工行） */
    @Value("${commProperties.ftp_sz_icbc_return_path_name}")
    private String icbcZlsPathName;

    @Resource
    private SfRepo sfRepo;

    @Override
    public void createFile(String dataName) {
        //先取文件
        logger.info("连接ftp服务器，获取深圳增量协议号文件.");
        String absolutePath = "/data/sf/file/";
        if (!StringTools.hasText(dataName)) {
            dataName = LocalDate.now().format(DateTimeFormatter.ofPattern("yyyyMMdd"));
        }
        SFTPUtil sftpUtil = new SFTPUtil(icbcZlsName, icbcZlsPwd,
                icbcZlsIpAddreess, icbcZlsPort);
        sftpUtil.login();
        // String documentEnd = "TB_EntrustDetail.txt";
        String fileAddress = absolutePath + "AGREE" + "_" + dataName + ".txt";
        String uploadFile = absolutePath + "AGREE" + "_" + dataName + "_RETURN.txt";
        logger.info("开始下载 深圳增量协议号文件." + fileAddress);
        try {
            sftpUtil.download(icbcZlsPathName, "AGREE" + "_" + dataName + ".txt", fileAddress);
        } catch (FileNotFoundException | SftpException e) {
            e.printStackTrace();
        }
        sftpUtil.logout();
        logger.info("深圳增量协议号文件下载成功.");
        //解析文件
        List<SfModel> agreementNums = Lists.newArrayList();
        List<Object> rows = TxtUtil.readTxtData(fileAddress);
        logger.info("协议号文件行数" +rows.size());
        for (int i = 1 ; i < rows.size() - 1; i++) {
            SfModel model = new SfModel();
            String o = String.valueOf(rows.get(i)).replace("[","").replace("]","");
            String[] split = o.split("\\|");
            model.setType(Integer.valueOf(split[0]));
            model.setAgreementId(split[1]);
            agreementNums.add(model);
        }
        //组装文件
        OutputStreamWriter out = null;
        // 获取当前的时间
        // String dateTime = DateUtils.date2ymdString(date);
        // String fileName = "TB_" + dateTime + "_SFUserinfo_Detail";
        try {
            out = new OutputStreamWriter(new FileOutputStream(uploadFile,
                    false), "GB2312");
            out.write("标识符" + "|" + "工行签约协议号" + "|" + "车牌号" + "|" + "车牌颜色"
                    + "|" + "证件号" + "|" + "客户姓名" + "|" + "部门" + "|" + "ETC卡号"
                    + "|" + "ETC卡状态" + "|" + "渠道名称" + "|" + "\r\n");
            int j = 0;
            for (int i = 0; i < agreementNums.size(); i++) {
                SfModel agreementNumBean = agreementNums.get(i);
                System.out.println(agreementNumBean.getAgreementId());
                Map<String, String> query1 = Maps.newHashMap();
                query1 = sfRepo.query1(agreementNumBean.getAgreementId());
                if (query1 == null || query1.size() == 0) query1 = sfRepo.query2(agreementNumBean.getAgreementId());
                if (query1 == null || query1.size() == 0) query1 = sfRepo.query3(agreementNumBean.getAgreementId());
                System.out.println(JSON.toJSON(query1));
                if (query1 != null && query1.size() > 0) {
                    j++;
                    String vehicleId = query1.get("vehicleId");
                    String[] s = vehicleId.split("_");
                    VehicleModel vehicle = getVehicleInfo(vehicleId);
                    if (StringTools.hasText(vehicle.getCardId())) vehicle.setCardStatus("1");
                    out.write("1" + "|" + query1.get("agreementNumber") + "|"
                            + vehicle.getVehiclePlate() + "|"
                            + vehicle.getVehiclePlateColor() + "|"
                            + query1.get("userIdNo") + "|"

                            + query1.get("userName") + "|" + query1.get("department") + "|"
                            + vehicle.getCardId() + "|" + vehicle.getCardStatus() + "|"
                            + "工总行直连模式三" + "|" + "\r\n");
                    logger.info("1" + "|" + query1.get("agreementNumber") + "|"
                            + vehicle.getVehiclePlate() + "|"
                            + vehicle.getVehiclePlateColor() + "|"
                            + query1.get("userIdNo") + "|"

                            + query1.get("userName") + "|" + query1.get("department") + "|"
                            + vehicle.getCardId() + "|" + vehicle.getCardStatus() + "|"
                            + "工总行直连模式三" + "|" );
                }
            }
            out.write("2" + "|" + j + "|" + "\r\n");
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
            } catch (Exception e2) {
                e2.printStackTrace();
            }
        }
        logger.info("组装文件结束");
        FileInputStream input = null;
        try {
            input = new FileInputStream(uploadFile);
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        }
        Map<String, InputStream> fileStream = new HashMap<String, InputStream>();
        dataName = dataName.substring(0,6) + "01";
        fileStream.put("AGREE" + "_" + dataName + "_RETURN.txt", input);
        sftpUtil.login();
        try {
            // 这是需要将追加的流水上传到工行的的流水
            sftpUtil.upload(icbcZlsPathName, null, fileStream);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            sftpUtil.logout();
        }
    }

    @Autowired
    private QtHttpTemplate qtHttpTemplate;

    private VehicleModel getVehicleInfo(String vehicleId) throws Exception {
        Map<String, String> map = new HashMap<String, String>();
        map.put("vehicleId", vehicleId);
        String data = qtHttpTemplate.execute(QtCodeEnum.QUERY_CAR_INFO, map);
        // 注, 按照接口去解析。返回不一定是 data
        logger.info(data);
        if (!StringTools.hasText(data)) {
            return new VehicleModel();
        }
        return JSONObject.parseObject(data, VehicleModel.class);
    }
}
